export declare type F<T> = (...args: any[]) => T;
export declare class Transaction {
    private static id;
    readonly id: number;
    readonly hint: string;
    readonly snapshot: number;
    readonly error: any;
    private _snapshot;
    private changeset;
    private pending;
    private sealed;
    private _error;
    private resultPromise;
    private resultResolve;
    private resultReject;
    private conflicts;
    private refreshments;
    constructor(hint?: string);
    static readonly current: Transaction | undefined;
    run<T>(func: F<T>, ...args: any[]): T;
    wrap<T>(func: F<T>): F<T>;
    commit(): void;
    sealToCommit(): Transaction;
    discard(error?: any): Transaction;
    waitForFinish(): Promise<void>;
    finished(): boolean;
    undo(): void;
    redo(): void;
    static run<T>(hint: string, func: F<T>, ...args: any[]): T;
    static runAsync<T>(hint: string, func: F<Promise<T>>, ...args: any[]): Promise<T>;
    private _run;
    private sealAndRethrowErrorIfNeeded;
    private static seal;
    private checkConflictsResolved;
    private tryResolveConflicts;
    private performCommit;
    private performDiscard;
    private acquirePromise;
    static _acquireSnapshotNumber(t: Transaction): number;
    static _changeset(t: Transaction): Map<Handle, Record>;
    static _wrap<T>(t: Transaction, c: Cache | undefined, inc: boolean, dec: boolean, func: F<T>): F<T>;
}
export declare class StoreParams {
    readonly name?: string;
    readonly idProperty?: string;
    readonly isObjectWithId?: (obj: object) => boolean;
    readonly twoWaySync?: (incomingPatches: object[], outgoingPatches: object[]) => void;
}
export declare class Store {
    readonly params: StoreParams;
    constructor(params?: StoreParams);
    static dispose(obj: object): void;
}
declare class Handle {
    private static id;
    readonly id: number;
    readonly store: Store;
    readonly proxy: any;
    private committed;
    private editable;
    private editors;
    constructor(store: Store, proxy: any);
    getCommitted(): Record;
    tryGetForRead(t: Transaction, c: Cache | undefined): Record | undefined;
    getForRead(): Record;
    acquireForEdit(prop: string): Record;
    tryAcquireForEdit(prop: string, value: any): Record | undefined;
    static commitOrDiscard(success: boolean, changeset: Map<Handle, Record>): void;
}
declare type PropertyMask<T, V> = {
    [P in keyof T]: V;
};
declare class Record {
    readonly handle: Handle;
    readonly previous: Record | undefined;
    readonly transaction: Transaction;
    readonly data: any;
    readonly changemask: PropertyMask<any, boolean>;
    readonly conflictmask: PropertyMask<any, Record>;
    readonly observers: PropertyMask<any, Set<Cache>>;
    readonly stalemask: PropertyMask<any, boolean>;
    constructor(handle: Handle, previous: Record | undefined, tran: Transaction, data: object);
    changed(prop: string, changed: boolean): void;
    freeze(): void;
    propUsedByCache(prop: string, cache: Cache | undefined): void;
    invalidateObservers(refreshments: Cache[]): number;
    determineConflicts(): boolean;
    invalidateObserversIfAny(prop: string, observers: Set<Cache> | undefined, refreshments?: Cache[]): number;
    hint(): string;
}
declare class MethodInfo {
    readonly isTran: boolean;
    readonly isAsync: boolean;
    readonly name: string;
    readonly body: (...args: any[]) => any;
    constructor(isTran: boolean, name: string, body: F<any>);
}
declare class Cache {
    readonly owner: Record;
    readonly method: MethodInfo;
    readonly wrapper: (...args: any[]) => any;
    readonly observables: PropertyMask<any, Set<Record>>;
    args: any[];
    value: any;
    invalidator: Record | undefined;
    stale: Record | undefined;
    error: any;
    latency: number;
    margin: number;
    constructor(owner: Record, method: MethodInfo, latency: number, margin: number);
    run<T>(func: F<T>, ...args: any[]): T;
    wrap<T>(func: F<T>): F<T>;
    ensureFresh(now: boolean): void;
    tryInvalidateBy(invalidator: Record, prop: string, refreshments?: Cache[]): number;
    private createWrapper;
    private renew;
    private renewAsync;
    hint(): string;
}
export interface Status {
    value: any;
    outdated: boolean;
    error: any;
    latency: number;
}
export declare function statusof(method: F<any>): Status;
export declare function autorenew(latency: number, method: F<any>, ...args: any[]): void;
export declare function dispose(obj: object): void;
export declare const transactional: typeof tran;
export declare function tran(target: object, prop?: string, descriptor?: TypedPropertyDescriptor<F<any>>): any;
export declare function cache(ctor: object, prop: string, descriptor: TypedPropertyDescriptor<F<any>>): any;
export {};
